package cc.fascinated.model.user;

import cc.fascinated.model.user.statistic.Statistic;
import cc.fascinated.platform.Platform;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.*;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @author Fascinated (fascinated7)
 */
@RequiredArgsConstructor
@Getter
@Setter
@ToString
@Document("user")
public class User {
    /**
     * The ID of the user.
     */
    @Id
    @JsonIgnore
    private final UUID id;

    /**
     * The username of the user.
     * <p>
     * Usually their Steam name.
     * </p>
     */
    @Indexed
    private String username;

    /**
     * The ID of the users steam profile.
     */
    @Indexed
    @JsonProperty("id")
    private String steamId;

    /**
     * Whether the user has logged into the website.
     * <p>
     * This is used to determine if we should track their profiles or not.
     * If they haven't logged in, we don't want to track their profiles.
     * </p>
     */
    @JsonIgnore
    public boolean hasLoggedIn;

    /**
     * The user's ScoreSaber account.
     */
    public ScoreSaberAccount scoresaberAccount;

    /**
     * The user's statistic history.
     */
    @JsonIgnore
    public Map<Platform.Platforms, Map<Date, Statistic>> statistics;

    /**
     * Gets the statistics for a platform.
     *
     * @param platform the platform to get the statistics for
     * @return the statistics
     */
    public Map<Date, Statistic> getStatistics(@NonNull Platform.Platforms platform) {
        if (this.statistics == null) {
            this.statistics = new HashMap<>();
        }
        return this.statistics.computeIfAbsent(platform, k -> new HashMap<>());
    }

    /**
     * Adds a statistic to the user's history.
     *
     * @param platform the platform to add the statistic for
     * @param date the date of the statistic
     * @param statistic the statistic to add
     */
    public void addStatistic(@NonNull Platform.Platforms platform, @NonNull Date date, @NonNull Statistic statistic) {
        this.getStatistics(platform).put(date, statistic);
    }
}
